/*

    qmailalizer - a qmail-send log analysis program

    Copyright (C) 2001  Pichler Wolfgang  (wpichler@dialog-gruppe.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.


*/

#include "parse.h"
#include "message.h"
#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <sys/wait.h>
#include <stdlib.h>
#include "tai64.h"

Parser::Parser ()
{
  LogWurzel = NULL;
  messages = new MHandler();
  config = new Config ();
  messages->loguser = config->loguser;
  images = new Images(config);
  results = new Results(messages,images,config);
  xmlinput = new XMLInput (config,messages,results);
  xmloutput = new XMLOutput ();
}

Parser::~Parser ()
{
  freefiles ();
  delete (messages);
  delete (xmloutput);
  delete (results);
  delete (images);
  delete (config);
}

void Parser::freefiles ()
{ LogFiles *aktElement;
  LogFiles *nextElement;
  
  aktElement = LogWurzel;
  while (aktElement != NULL)
  {
    nextElement = aktElement->next;
    if (aktElement->filename != NULL)
      free (aktElement->filename);
    free (aktElement);
    aktElement = nextElement;
  }
}

int Parser::start ()
{ LogFiles *aktElement;

  // Die bereits berechneten Ergbenisse laden
  xmlinput->LoadResults ();
  LogWurzel = xmlinput->lWurzel;
  logfiles ();
  if (LogWurzel != NULL)
  {
    // Es existieren also Dateien
    aktElement = LogWurzel;
    while (aktElement != NULL)
    {
      if (!aktElement->processed)
        parsefile (aktElement);
      else
        printf ("File: %s is already processed.\n",aktElement->filename);
      aktElement = aktElement->next;
    }
    return 1;
  }
  else
    return 0; 
}

int Parser::parsefile (LogFiles *file)
{
  char *zeile;
  int local,maxlocal,remote,maxremote;
  char temp[200];
  char temp1[5];
  char *temp2;
  unsigned long dnr,msgnr;
  char *recipient;
  char *localremote;
  char *dstate;
  long bytes;
  char *sender;
  char tstamp[30];
  struct tm *datum;
  struct tm *tempdatum;
  FILE *f;
  char cfilename[256];
  char line[512];
  
  printf ("Parsing file: %s\n",file->filename);
  strcpy (cfilename,config->logdir);
  strcat (cfilename,file->filename);
  f = fopen (cfilename,"r");
  datum = new(struct tm);
  while (!feof(f))
  {
    // In jeder zeile steht am Anfang mal irgendwas das genau 26 Zeichen inkl space
    fgets (line,512,f);
    zeile = line;
    strncpy (tstamp,zeile,26);
    tempdatum = converttai64(tstamp);
    memcpy (datum,tempdatum,sizeof(struct tm));
    zeile = zeile + 26;
    zeile[strlen(zeile)-1] = 0;
    // jetzt berprfen was am Anfang steht
    if (!strncmp(zeile,"status",strlen("status")))
    {
      // Nun die einzelnen Status Werte rausholen
      // also erst einmal um den status noch vorn gehen
      zeile = zeile + strlen("status: ");
      if (strncmp(zeile,"local",5)==0)
      {
        zeile = zeile + strlen("local");
        // Ab dieser Position steht dann xxx/xxx - also mal den String bis zum nchsten leerzeichen rausholen 
        strncpy(temp,zeile,strchr(zeile,' ')-zeile+1);
        // jetzt aus xxx/xxx die beiden werte rausfiltern
        strcpy(temp1,temp);
        strtok(temp1,"/");
        temp2 = &temp[strlen(temp1)+1];
        local = atoi(temp1);
        maxlocal = atoi(temp2);
        // Jetzt kommen die beiden remote Werte
        zeile = strrchr(zeile,' ')+1;
        strcpy(temp1,zeile);
        strtok(temp1,"/");
        temp2 = &temp[strlen(temp1)+1];
        remote = atoi(temp1);
        maxremote = atoi(temp2);
        //Die ermittelten Werte jetzt eintragen
        messages->deliveriestatus (local,maxlocal,remote,maxremote);
      }
    }
    else if (!strncmp(zeile,"starting delivery",strlen("starting delivery")))
    {
      zeile = zeile + strlen("starting delivery ");
      // die delivery nr geht bis zum :
      strncpy (temp,zeile,strpos(zeile,':'));
      temp[strpos(zeile,':')] = 0;
      dnr = strtol(temp,(char **)NULL,10);
      zeile = zeile + strlen(temp)+6;
      strncpy (temp,zeile,strpos(zeile,' '));
      temp[strpos(zeile,' ')] = 0;
      msgnr = atoi(temp);
      zeile = zeile + strlen(temp)+4;
      strncpy (temp,zeile,strpos(zeile,' '));
      temp[strpos(zeile,' ')] = 0;
      localremote = temp;
      zeile = zeile + strlen(temp) + 1;
      recipient = zeile;
      messages->newdeliverie (dnr,msgnr,recipient,localremote,datum);
    }
    else if (!strncmp(zeile,"delivery",strlen("delivery")))
    {
      zeile = zeile + strlen("delivery ");
      strncpy (temp,zeile,10);
      strtok (temp,":");
      dnr = atoi(temp);
      zeile = zeile + strlen(temp) + 2;
      strncpy (temp,zeile,50);
      strtok (temp,":");
      zeile = zeile + strlen(temp) + 2;
      dstate = temp;
      messages->deliveriestate (dnr,dstate,zeile,datum);
    }
    else if (!strncmp(zeile,"end msg",strlen("end msg")))
    {
      zeile = zeile + strlen("end msg ");
      // die msgnr rausfiltern
      msgnr = atoi(zeile);
      messages->endmessage (msgnr,datum);
    }
    else if (!strncmp(zeile,"info msg",strlen("info msg")))
    {
      zeile = zeile + strlen("info msg ");
      strncpy (temp,zeile,strpos(zeile,':'));
      temp[strpos(zeile,':')] = 0;
      msgnr = atoi(temp);
      zeile = zeile + strlen(temp) + 8;
      strncpy (temp,zeile,strpos(zeile,' '));
      temp[strpos(zeile,' ')] = 0;
      bytes = atoi(temp);
      zeile = zeile + strlen(temp) + 7;
      strncpy (temp,zeile,strpos(zeile,'>'));
      temp[strpos(zeile,'>')] = 0;
      sender = temp;
      messages->infomessage(msgnr,bytes,sender,datum);
    }
  }
  free (datum);
  file->processed = true;
  return 0;
}

bool Parser::Processed(char *name)
{ LogFiles *aktFile;
  
  aktFile = LogWurzel;
  while (aktFile != NULL) {
    if (strcmp(aktFile->filename,name) == 0) break;
    aktFile = aktFile->next;
  }
  if (aktFile != NULL) return true;
  else return false;
}

int Parser::logfiles ()
{ DIR *temp;
  struct dirent *direntp;
  LogFiles *nElement=NULL;
  LogFiles *aktElement;
  int files=0;
  bool usable;
  struct tm *datum;
  time_t dwert,dwert1;
  char *tempfile;
  
  aktElement = LogWurzel;
  // Ans Ende der Liste gehen
  if (aktElement != NULL)
    while (aktElement->next != NULL) aktElement = aktElement->next;
  temp = opendir (config->logdir);
  while((direntp = readdir(temp)) != NULL)
  {
    // Alle Dateien die die einem @ beginnen -> verwenden
    // ohne dem current file  
    usable = false;
    if (direntp->d_name[0] == '@')
    {
      files++;
      // Hier berprfen ob das file nicht schon mal verarbeitet wurde
      if (!Processed(direntp->d_name)) {
        nElement = new(LogFiles);
        nElement->filename = (char *)malloc(strlen(direntp->d_name)+1);
        strcpy (nElement->filename,direntp->d_name);
        nElement->processed = false;
        usable = true;
      }
      else
        usable = false;
    }
    if (usable)
    {
      nElement->next = NULL;
      nElement->prev = NULL;
      if (aktElement == NULL)
      {
        aktElement = nElement;
        LogWurzel = aktElement;
      }
      else
      {
        aktElement->next = nElement;
	nElement->prev = aktElement;
        aktElement = aktElement->next;
      }
    }
  }
  closedir(temp);	// causes a segmentation fault <- don't know why
  // Jetzt gehrt die Liste auch noch sortiert nach dem Datum
  if (LogWurzel != NULL) {
    aktElement = LogWurzel;
    while (aktElement != NULL) {
      datum = converttai64(aktElement->filename);
      dwert = mktime(datum);

      nElement = aktElement->next;
      while (nElement != NULL) {
        datum = converttai64(nElement->filename);
        dwert1 = mktime(datum);
        if (dwert1 < dwert) {
	  // Es gehrt vertauscht
          tempfile = aktElement->filename;
	  aktElement->filename = nElement->filename;
	  nElement->filename = tempfile;
	}
        nElement = nElement->next;
      }
      aktElement = aktElement->next;
    }
  }
  return files;
}

// Sammelt alle Daten die gesichert werden mssen
void Parser::saveresults ()
{ LogFiles *aktfile;
  time_t zeit;
  char *ptime;
  Message *aktmessage;
  
  // Hier werden die bereits verarbeiteten Dateien registriert
  aktfile = LogWurzel;
  ptime = (char *)malloc(100);
  while (aktfile != NULL) {
    zeit = time(NULL);
    ptime = ctime(&zeit);
    ptime[strlen(ptime)-1] = 0;
    xmloutput->AddFile (aktfile->filename,aktfile->processed,ptime);
    aktfile = aktfile->next;
  }
  // Hier nun die noch nicht abgeschlossenen Nachrichten sammeln
  aktmessage = messages->mWurzel;
  while (aktmessage != NULL) {
    if (!aktmessage->endmsg) {
      xmloutput->AddMessage (aktmessage);
    }
    aktmessage = aktmessage->next;
  }
  results->save_results (xmloutput);
}

int Parser::save ()
{
  saveresults ();
  xmloutput->SaveResult (config->xmlfile);
  return 0;
}
